#lang racket

;; ------------------------------------------------------------------------
;; This sequence of examples leads to a reliable bank account
;; ------------------------------------------------------------------------

(define balance 100)

;; ------------------------------------------------------------------------

;; (define withdraw
;;   (lambda (amount)
;;     (if (>= balance amount)         ;; balance is a free variable!!
;;         (begin (define balance (- balance amount))
;;                balance)
;;         (error "Insufficient funds" balance))))

;; ------------------------------------------------------------------------

;; (define withdraw
;;   (lambda (amount)
;;     (if (>= balance amount)
;;         (begin (set! balance (- balance amount))
;;                balance)
;;         (error "Insufficient funds: " balance))))

;; ------------------------------------------------------------------------

;; (define withdraw
;;   (lambda (amount)
;;     (let ((balance 100))
;;       (if (>= balance amount)         ;; now balance is bound
;;           (begin (set! balance (- balance amount))
;;                  balance)
;;           (error "Insufficient funds" balance)))))

;; ------------------------------------------------------------------------
;; This example uses a closure to encapsulate a data object
;; ------------------------------------------------------------------------

;; (define withdraw
;;   (let ((balance 100))
;;     (lambda (amount)
;;       (if (>= balance amount)        ;; balance is still bound!!
;;           (begin (set! balance (- balance amount))
;;                  balance)
;;           (error "Insufficient funds: " balance)))))

;; ------------------------------------------------------------------------
;; This example uses a lambda to create a closure
;; ------------------------------------------------------------------------

;; (define make-withdraw
;;   (lambda (balance)
;;     (lambda (amount)
;;       (if (>= balance amount)        ;; balance is still bound, but
;;           (begin                     ;; to a new object on each call!
;;              (set! balance (- balance amount))
;;              balance)
;;           (error "Insufficient funds: " balance)))))

;; ------------------------------------------------------------------------
;; A silly little function used in Session 23
;; ------------------------------------------------------------------------

(define embezzle
  (lambda (amount)
    (if (>= balance amount)
        (set! balance (- balance amount))
        (set! balance 0))))

;; ------------------------------------------------------------------------

